import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * An example application that uses double buffering.
 * 
 * @author Simon Parsons
 *
 * This is based on the example from Niemeyer and Knudsen
 * "Learning Java", pages 718 and 719.
 * 
 * This class does the hard work, especially the double 
 * buffering that this example is intended to showcase.
 */
class DrawPad extends JComponent{
	Image image;
	Graphics2D graphics2D;
	int currentX, currentY, oldX, oldY;
	boolean justCleared;
	
	public DrawPad(){
		justCleared = true;
		setDoubleBuffered(false);
		// Another anonymous listener to handle the mouse button
		// being pressed.
		addMouseListener(new MouseAdapter(){
			public void MousePressed(MouseEvent e){
				oldX = e.getX();
				oldY = e.getY();
			}
		});
		// The listener that responds to the mouse being dragged
		// draws to an off-screen buffer graphics2D
		addMouseMotionListener(new MouseMotionAdapter(){
			public void mouseDragged(MouseEvent e){
				currentX = e.getX();
				currentY = e.getY();
				// Draw a line in the offscreen buffer. Checking justCleared makes sure
				// we don't draw spurious lines.
				if(graphics2D != null && !justCleared){
					graphics2D.drawLine(oldX, oldY, currentX, currentY);
				}
				// Now call paint
				repaint();
				oldX = currentX;
				oldY = currentY;
				justCleared = false;
			}
		});
	}
	
	// The paint method makes a new off-screen buffer if none exists and
	// otherwise copies the offscreen buffer to the screen.
	public void paintComponent(Graphics g){
		if (image == null){
			image = createImage(getSize().width, getSize().height);
			graphics2D = (Graphics2D)image.getGraphics();
			graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
			clear();
		}
		g.drawImage(image,  0, 0, null);
	}
	
	// Clear the offscreen buffer and then repaint to transfer its contents
	// onto the screen.
	public void clear(){
		graphics2D.setPaint(Color.white);
		graphics2D.fillRect(0, 0, getSize().width, getSize().height);
		graphics2D.setPaint(Color.black);
		justCleared = true;
		repaint();
	}
}
